#ifndef SCOPE_QUERY_H_
#define SCOPE_QUERY_H_

#include <unity/scopes/SearchQueryBase.h>
#include <unity/scopes/ReplyProxyFwd.h>
#include <unity/scopes/OptionSelectorFilter.h>

namespace scope {

/**
 * Represents an individual query.
 *
 * A new Query object will be constructed for each query. It is
 * given query information, metadata about the search, and
 * some scope-specific configuration.
 */
class Query: public unity::scopes::SearchQueryBase {
public:
    Query(const unity::scopes::CannedQuery &query,
          const unity::scopes::SearchMetadata &metadata, const std::string &scopedir);

    ~Query() = default;

    void cancelled() override;

    void run(const unity::scopes::SearchReplyProxy &reply) override;

private:
    std::vector<std::string> generate_product_variants(const unity::scopes::OptionSelectorFilter::SPtr& filter);
    void generate_results(
            unity::scopes::SearchReplyProxy const& reply,
            std::string const& book_format,
            std::set<std::string> const& book_categories,
            double min_price,
            double max_price,
            double rating);

    std::string scope_dir_;
};

}

#endif // SCOPE_QUERY_H_
